package ab;

import static ab.IconUtil.getLargeIcon;
import static ab.IconUtil.getSmallImage;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import ab.AddressDialog.Option;
import ab.table.AddressTable;

/**
 * Visual representation of an address book. This is also the application's main
 * class.
 * 
 * @author ahoefer
 * 
 */
public class AddressBook extends JFrame {

	private static final long serialVersionUID = 1738422841479223075L;

	private AddressTable table = new AddressTable();

	/**
	 * Creates a new address book.
	 */
	public AddressBook() {
		setTitle("SWT 1 - Adressbuch");
		setIconImage(getSmallImage("kit"));
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(table);
		contentPane.add(scrollPane, BorderLayout.CENTER);
		contentPane.add(createControl(), BorderLayout.PAGE_START);

		pack();
		setLocationRelativeTo(null);
	}

	private JToolBar createControl() {
		JToolBar control = new JToolBar();
		control.add(new AddAddressAction());
		control.add(new EditAddressAction());
		control.add(new RemoveAddressAction());
		control.add(new JToolBar.Separator());
		control.add(new ExitAction());
		return control;
	}

	private class AddAddressAction extends AbstractAction {

		private static final long serialVersionUID = 5036490585785108338L;

		public AddAddressAction() {
			super("Hinzufuegen");
			putValue(LARGE_ICON_KEY, getLargeIcon("add"));
			putValue(SHORT_DESCRIPTION, "Hinzufuegen");
		}

		public void actionPerformed(ActionEvent e) {
			AddressDialog dialog = new AddressDialog(AddressBook.this);
			dialog.setVisible(true);
			if (dialog.getSelectedOption() == Option.SAVE) {
				table.addAddress(dialog.getAddress());
			}
		}
	}

	private class EditAddressAction extends AbstractAction {

		private static final long serialVersionUID = -1200406722700964528L;

		public EditAddressAction() {
			super("Bearbeiten");
			putValue(LARGE_ICON_KEY, getLargeIcon("edit"));
			putValue(SHORT_DESCRIPTION, "Bearbeiten");
		}

		public void actionPerformed(ActionEvent e) {
			Address address = table.getSelectedAddress();
			if (address == null) {
				return;
			}
			AddressDialog dialog = new AddressDialog(address, AddressBook.this);
			dialog.setVisible(true);
			if (dialog.getSelectedOption() == Option.SAVE) {
				table.updateSelectedAddress(dialog.getAddress());
			}
		}
	}

	private class RemoveAddressAction extends AbstractAction {

		private static final long serialVersionUID = -6681795431514148118L;

		public RemoveAddressAction() {
			super("Entfernen");
			putValue(LARGE_ICON_KEY, getLargeIcon("remove"));
			putValue(SHORT_DESCRIPTION, "Entfernen");
		}

		public void actionPerformed(ActionEvent e) {
			table.removeSelectedAddress();
		}
	}

	private class ExitAction extends AbstractAction {

		private static final long serialVersionUID = 2727566117954481029L;

		public ExitAction() {
			super("Beenden");
			putValue(LARGE_ICON_KEY, getLargeIcon("exit"));
			putValue(SHORT_DESCRIPTION, "Beenden");
		}

		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}
	}

	/**
	 * Starts the application.
	 * 
	 * @param args
	 *            Arguments will be ignored.
	 */
	public static void main(String[] args) {
		AddressBook ab = new AddressBook();
		ab.setVisible(true);
	}
}
