package ab;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * This dialog allows editing of an existing address as well as data collection
 * for a new address.
 * 
 * @author ahoefer
 * 
 */
public class AddressDialog extends JDialog {

	private static final long serialVersionUID = -3531727509798660627L;

	private JComboBox category = new JComboBox(Category.values());
	private JTextField firstname = new JTextField(10);
	private JTextField lastname = new JTextField(10);
	private JTextField address1 = new JTextField(10);
	private JTextField address2 = new JTextField(10);
	private JTextField zipCode = new JTextField(4);
	private JTextField city = new JTextField(10);
	private JTextField country = new JTextField(10);
	private JTextField eMail = new JTextField(10);
	private JTextField landline = new JTextField(10);
	private JTextField mobile = new JTextField(10);

	private Address address;

	private Option selected = Option.CANCEL;

	/**
	 * Creates a modal dialog with a new empty address.
	 * 
	 * @param owner
	 *            The Frame from which the dialog is displayed.
	 */
	public AddressDialog(JFrame owner) {
		this(null, owner);
	}

	/**
	 * Creates a modal dialog to edit the given address.
	 * 
	 * @param address
	 *            The address to edit. If the address is null a new address will
	 *            be created.
	 * @param owner
	 *            The Frame from which the dialog is displayed.
	 */
	public AddressDialog(Address address, Frame owner) {
		super(owner, true);
		if (address == null) {
			this.address = new Address();
			setTitle("Adresse hinzufuegen");
		} else {
			this.address = address;
			setTitle("Adresse bearbeiten");
		}
		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(createControlPanel(), BorderLayout.PAGE_END);
		contentPane.add(createAddressPanel(), BorderLayout.CENTER);
		loadAddress();

		pack();
		setLocationRelativeTo(owner);
	}

	private void loadAddress() {
		category.setSelectedItem(address.getCategory());
		firstname.setText(address.getFirstname());
		lastname.setText(address.getLastname());
		address1.setText(address.getAddress1());
		address2.setText(address.getAddress2());
		zipCode.setText(address.getZipCode());
		city.setText(address.getCity());
		country.setText(address.getCounty());
		eMail.setText(address.getEMail());
		landline.setText(address.getLandline());
		mobile.setText(address.getMobile());
	}

	private void saveAddress() {
		address.setCategory((Category) category.getSelectedItem());
		address.setFirstname(firstname.getText());
		address.setLastname(lastname.getText());
		address.setAddress1(address1.getText());
		address.setAddress2(address2.getText());
		address.setZipCode(zipCode.getText());
		address.setCity(city.getText());
		address.setCounty(country.getText());
		address.setEMail(eMail.getText());
		address.setLandline(landline.getText());
		address.setMobile(mobile.getText());
	}

	private JPanel createControlPanel() {
		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout(FlowLayout.TRAILING, 4, 4));
		JButton save = new JButton(new SaveAction());
		JButton cancel = new JButton(new CancelAction());
		controlPanel.add(save);
		controlPanel.add(cancel);
		return controlPanel;
	}

	private JPanel createAddressPanel() {
		category.setRenderer(new CategoryComboBoxCellRenderer());

		JPanel addressPanel = new JPanel();
		addressPanel.setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints(0,
				GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0,
				GridBagConstraints.BASELINE_LEADING, GridBagConstraints.NONE,
				new Insets(2, 2, 2, 2), 0, 0);

		addressPanel.add(new JLabel("Kategorie"), constraints);
		addressPanel.add(new JLabel("Vorname:"), constraints);
		addressPanel.add(new JLabel("Nachname:"), constraints);
		addressPanel.add(new JLabel("Adresse 1:"), constraints);
		constraints.gridy = 5;
		addressPanel.add(new JLabel("PLZ:"), constraints);
		constraints.gridy = GridBagConstraints.RELATIVE;
		addressPanel.add(new JLabel("Land:"), constraints);
		addressPanel.add(new JLabel("E-Mail:"), constraints);
		addressPanel.add(new JLabel("Telefon:"), constraints);
		addressPanel.add(new JLabel("Mobil:"), constraints);

		constraints.gridx = 1;
		constraints.gridwidth = 3;
		constraints.weightx = 1.0;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		addressPanel.add(category, constraints);
		addressPanel.add(firstname, constraints);
		addressPanel.add(lastname, constraints);
		addressPanel.add(address1, constraints);
		addressPanel.add(address2, constraints);
		constraints.gridwidth = 1;
		// keep the ratio of 4:10 between zipCode and city textfield constant.
		constraints.weightx = 4.0;
		addressPanel.add(zipCode, constraints);
		constraints.gridwidth = 3;
		constraints.weightx = 1.0;
		addressPanel.add(country, constraints);
		addressPanel.add(eMail, constraints);
		addressPanel.add(landline, constraints);
		addressPanel.add(mobile, constraints);

		constraints.gridwidth = 1;
		constraints.gridx = 2;
		constraints.gridy = 5;
		constraints.fill = GridBagConstraints.NONE;
		constraints.weightx = 0.0;
		addressPanel.add(new JLabel("Stadt:"), constraints);
		constraints.gridx = 3;
		constraints.gridy = 5;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		// keep the ratio of 4:10 between zipCode and city textfield constant.
		constraints.weightx = 10.0;
		addressPanel.add(city, constraints);

		return addressPanel;
	}

	/**
	 * The address displayed by this dialog. The address is only altered if the
	 * user chooses to save.
	 * 
	 * @return Never null.
	 */
	public Address getAddress() {
		return address;
	}

	/**
	 * Returns the option chosen by the user.
	 * 
	 * @return Either SAVE or CANCEL, never null.
	 */
	public Option getSelectedOption() {
		return selected;
	}

	/**
	 * Represents the options the user can choose from.
	 * 
	 * @author ahoefer
	 * 
	 */
	public enum Option {
		/**
		 * Save the new/edited address data.
		 */
		SAVE,

		/**
		 * Cancel and discard the new/edited address data.
		 */
		CANCEL;
	}

	private class SaveAction extends AbstractAction {

		private static final long serialVersionUID = 2195322695813010605L;

		public SaveAction() {
			super("Speichern");
		}

		public void actionPerformed(ActionEvent e) {
			AddressDialog.this.selected = Option.SAVE;
			AddressDialog.this.saveAddress();
			AddressDialog.this.setVisible(false);
		}
	}

	private class CancelAction extends AbstractAction {

		private static final long serialVersionUID = -4164529031513463085L;

		public CancelAction() {
			super("Abbrechen");
		}

		public void actionPerformed(ActionEvent e) {
			AddressDialog.this.setVisible(false);
		}
	}
}
