package ab;

import static ab.IconUtil.getEmptyIcon;
import static ab.IconUtil.getSmallIcon;

import javax.swing.Icon;

/**
 * Categories for the addresses. This enum also encapsulates the visual
 * appearance of the categories.
 * 
 * @author ahoefer
 * 
 */
public enum Category {

	/**
	 * Addresses of people from work.
	 */
	WORK("Arbeit", getSmallIcon("work"), ColorScheme.RED),

	/**
	 * Addresses of family members.
	 */
	FAMILY("Familie", getSmallIcon("family"), ColorScheme.GREEN),

	/**
	 * Addresses of friends.
	 */
	FRIENDS("Freunde", getSmallIcon("friends"), ColorScheme.BLUE),

	/**
	 * Addresses that fit into none of the other categories.
	 */
	NONE("Keine", getEmptyIcon(), ColorScheme.YELLOW);

	private final String text;

	private final Icon icon;

	private final ColorScheme colorScheme;

	private Category(String text, Icon icon, ColorScheme colorScheme) {
		this.text = text;
		this.icon = icon;
		this.colorScheme = colorScheme;
	}

	/**
	 * Gets the textual representation of the category.
	 * 
	 * @return Never null.
	 */
	public String getText() {
		return text;
	}

	/**
	 * Gets the icon for the category.
	 * 
	 * @return Never null.
	 */
	public Icon getIcon() {
		return icon;
	}

	/**
	 * Gets the color scheme for the category.
	 * 
	 * @return Never null.
	 */
	public ColorScheme getColorScheme() {
		return colorScheme;
	}

	/**
	 * Overrides Enum.toString().
	 * 
	 * @return The same as getText().
	 * @see Category#getText()
	 * @see Enum#toString()
	 */
	@Override
	public String toString() {
		return text;
	}

}
